% Programm zur Loesung der Aufgabe 10.1
% mit Dirichlet- und Neumann-RB, iterative Loesung
% input: n,m Stuetzpunkte in x- und y-Richtung 
% output: Loesung u(x,y) auf dem Gitter x,y, 
%         udiff Abstand zweier Loesungen, it Iterationszahl
% Aufruf: [u,x,y,udiff,it] = aufg91i(n,m,omega,epsilon,itmax);
function [u,x,y,udiff,it] = aufg91i(n,m,omega,epsilon,itmax);
% Materialdaten, Quellen
la = 3.0; ff=0;
% Gebeitslaenge, Gebietshoehe
ll = 2; hh = 1;
% Dimensionierung des Gitters
dx = ll/n; dy = hh/m;
h = dx; k = dy;
% Gitterinitialisierung
x(1) = dx/2; y(1) = dy/2;
for i=1:n
  x(i) = (i-1)*dx+x(1);
end
for j=1:m
  y(j) = (j-1)*dy+y(1);
end
% Startfeld
for i=1:n
  for j=1:m
    u(i,j) = 1.0;
  end
end 
% Konstanten
h2=h^2; k2=k^2; dx2=2*dx; dy2=2*dy;
udiff = 1; unorm = 1; it = 0;
% Randwerte
uw=1; q1=0; q2=0; q3=0;
% Aufbau der Matrix und der rechten Seite
for i=1:n
    for j=1:m
      ind = i+(j-1)*n;
      vm = (x(i)-0.5*dx)^2; vp = (x(i)+0.5*dx)^2; 
      wm = (1-y(j)+0.5*dy)^2; wp = (1-y(j)-0.5*dy)^2;
      if (1 < i && i < n && 1 < j && j < m )
        a(ind,ind) = 2*la/h2+2*la/k2 + (vp-vm)/dx2 + (wp-wm)/dy2;
        a(ind,ind+1) = -(la/h2 - vp/dx2);
        a(ind,ind-1) = -(la/h2 + vm/dx2);
        a(ind,ind+n) = -(la/k2 - wp/dy2);
        a(ind,ind-n) = -(la/k2 + wm/dy2);
        ff1(ind) = ff;
      end
      if (i == 1 && 1 < j && j < m )
        a(ind,ind) = 3*la/h2+2*la/k2 + vp/dx2 + (wp-wm)/dy2;
        a(ind,ind+1) = -(la/h2 - vp/dx2);
        a(ind,ind+n) = -(la/k2 - wp/dy2);
        a(ind,ind-n) = -(la/k2 + wm/dy2);
        ff1(ind) = ff+vm*uw/h+la*2*uw/h2;
      end
      if (1 < i && i < n && j == 1 )
        a(ind,ind) = 2*la/h2+la/k2 + (vp-vm)/dx2 + wp/dy2 -wm/dy;
        a(ind,ind+1) = -(la/h2 - vp/dx2);
        a(ind,ind-1) = -(la/h2 + vm/dx2);
        a(ind,ind+n) = -(la/k2 - wp/dy2);
        ff1(ind) = ff+wm*q1*k/dy2+la*q1/k;
      end
      if (i == n && 1 < j && j < m )
        a(ind,ind) = la/h2+2*la/k2 + vp/dx -vm/dx2 + (wp-wm)/dy2;
        a(ind,ind-1) = -(la/h2 + vm/dx2);
        a(ind,ind+n) = -(la/k2 - wp/dy2);
        a(ind,ind-n) = -(la/k2 + wm/dy2);
        ff1(ind) = ff-vp*q2/2+la*q2/h;
      end
      if (1 < i && i < n && j == m )
        a(ind,ind) = 2*la/h2+la/k2 + (vp-vm)/dx2 + wp/dy - wm/dy2;
        a(ind,ind+1) = -(la/h2 - vp/dx2);
        a(ind,ind-1) = -(la/h2 + vm/dx2);
        a(ind,ind-n) = -(la/k2 + wm/dy2);
        ff1(ind) = ff-wp*q3/2+la*q3/k;
      end
      if (i == 1 && j == 1 )
        a(ind,ind) = 3*la/h2+la/k2 + vp/dx2 + wp/dy2 - wm/dy;
        a(ind,ind+1) = -(la/h2 - vp/dx2);
        a(ind,ind+n) = -(la/k2 - wp/dy2);
        ff1(ind) = ff+vm*uw/h+wm*q1/2+la*2*uw/h2+la*q1/k;
      end
      if (i == 1 && j == m )
        a(ind,ind) = 3*la/h2+la/k2 + vp/dx2 + wp/dy - wm/dy2;
        a(ind,ind+1) = -(la/h2 - vp/dx2);
        a(ind,ind-n) = -(la/k2 + wm/dy2);
        ff1(ind) = ff + vm*uw/h - wp*q3/2 + la*2*uw/h2 +la*q3/k;
      end
      if (i == n && j == 1 )
        a(ind,ind) = la/h2+la/k2 + vp/dx - vm/dx2 + wp/dy2 - wm/dy;
        a(ind,ind-1) = -(la/h2 + vm/dx2);
        a(ind,ind+n) = -(la/k2 - wp/dy2);
        ff1(ind) = ff-vp*q2/2+wm*q1/2+la*q2/h+la*q1/k;
      end
      if (i == n && j == m )
        a(ind,ind) = la/h2+la/k2 + vp/dx - vm/dx2 + wp/dy - wm/dy2;
        a(ind,ind-1) = -(la/h2 + vm/dx2);
        a(ind,ind-n) = -(la/k2 + wm/dy2);
        ff1(ind) = ff-vp*q2/2-wp*q3/2+la*q2/h+la*q3/k;
      end
    endfor
endfor
% SOR-Iteration
while (sqrt(udiff/unorm) > epsilon && it < itmax)
  ualt = u;
  udiff
  it
%
  for i=1:n
    for j=1:m
      ind = i+(j-1)*n;
      if (1 < i && i < n && 1 < j && j < m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-1)*u(i-1,j) ...
                  -a(ind,ind+n)*u(i,j+1)-a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      end
      if (i == 1 && 1 < j && j < m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind+n)*u(i,j+1) ...
                  -a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      end
      if (1 < i && i < n && j == 1 )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-1)*u(i-1,j) ...
                  -a(ind,ind+n)*u(i,j+1))/a(ind,ind);
      end
      if (i == n && 1 < j && j < m )
        u(i,j)=(ff1(ind)-a(ind,ind-1)*u(i-1,j)-a(ind,ind+n)*u(i,j+1) ...
                  -a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      end
      if (1 < i && i < n && j == m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-1)*u(i-1,j) ...
                  -a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      end
      if (i == 1 && j == 1 )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind+n)*u(i,j+1))/a(ind,ind);
      end
      if (i == 1 && j == m )
        u(i,j)=(ff1(ind)-a(ind,ind+1)*u(i+1,j)-a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      end
      if (i == n && j == 1 )
        u(i,j)=(ff1(ind)-a(ind,ind-1)*u(i-1,j)-a(ind,ind+n)*u(i,j+1))/a(ind,ind);
      end
      if (i == n && j == m )
        u(i,j)=(ff1(ind)-a(ind,ind-1)*u(i-1,j)-a(ind,ind-n)*u(i,j-1))/a(ind,ind);
      end
    end
  end
  udiff = 0.0; unorm = 0.0;
  for i=1:n
    for j=1:m
      u(i,j) = omega*u(i,j)+(1-omega)*ualt(i,j);
      udiff = udiff + (u(i,j)-ualt(i,j))^2;
      unorm = unorm + ualt(i,j)^2;
    end
  end
  if (unorm == 0.0)
    unorm = 1;
  end
  it = it + 1;
end
end
