% Programm loesung113.m  
% Milstein-Methode zur numerischen Loesung von
%
% SDE: dX = (alpha + mu*X) dt + (peta + sigma*X) dW, X(0) = Xzero ,
% alpha = 2 , mu = -3 , peta = 1 , sigma = 1 und Xzero = 1 .
%
% Mittelwert von M Loesungspfaden
% der numerischen Loesung im Vergleich zur exakten Loesung
%
% Wiener-Prozess ueber [0,1], dt = 2^(-8).
% Milstein auf einem Gitter mit Dt = R*dt (R Zweierpotenz)
randn('state',100) % random seeding
alpha = 2 ; mu = -3 ; peta = 1 ; sigma = 1 ; Xzero = 1 ; % Problemparameter
T = 1 ; N = 2^8 ; dt = 1/N; M = 100; %  
R = 4 ; Dt = R*dt ; L = N/R; % L Schritte der numerischen Verfahren Dt = R*dt
dW = zeros(M,N); W = zeros(M,N); Xtrue = zeros(1,N); % Pre-Allokierung
dW = sqrt(dt)*randn(M,N) ; % Inkremente des Wiener-Prozesses
W = cumsum(dW,2) ; % Wiener-Prozess
Uemean = zeros(1,N); Unmean = zeros(1,L); % Pre-Allokierung
s = 0;
while (s < M)
   s = s + 1;
   dw = dW(s,:);
   w = W(s,:);
%  Aufbau des s-ten Pfades der exakten Loesung
   j = 0;
   while (j < N)
     j = j + 1;
     ct1 = 0;
     k = 0;
     while (k < j)
       k = k + 1;
       summa = exp( (0.5*sigma^2-mu)*k*dt - sigma*w(k) );
       ct1 = ct1 + summa*dt;
       ct2(k) = exp( (0.5*sigma^2-mu)*k*dt - sigma*w(k) );
     end
     wito = ito_integral_gb(ct2(1:j),dw(1:j),j);
     Xtrue(j) = ( Xzero+exp(sigma*w(1))*( (alpha-peta*sigma)*ct1+peta*wito ) )...
              *exp( (mu-0.5*sigma^2)*j*dt+sigma*(w(j)-w(1)) );
   end
   Uemean = Uemean + Xtrue;
%  Berechnung des s-ten Pfades des numerischen Milstein-Loesung
   Yem = zeros(1,L) ; % Pre-Allokierung
   Ytemp = Xzero ;
   j = 0;
   while (j < L)
     j = j + 1;
     sWinc = sqrt(Dt)*sign(sum(dw(R*(j-1)+1:R*j))) ;
     Ytemp = Ytemp+Dt*(alpha+mu*Ytemp)+(peta+sigma*Ytemp)*sWinc...
                +0.5*sigma*(peta+sigma*Ytemp)*(sWinc^2-Dt);
     Yem(j) = Ytemp ;   %   schwache Milstein-Loesung
   end
   Unmean = Unmean + Yem;
end
Uemean = Uemean/M;
Unmean = Unmean/M;
plot([0:dt:T],[Xzero,Uemean],'b--'), hold on
plot([0:Dt:T],[Xzero,Unmean],'r--*'), hold off
xlabel('t','FontSize',16); ylabel('X_{mean}','FontSize',16);
for j=1:L
   Uem(j) = Uemean(R*(j-1)+1);
end 
error = norm(Uem-Unmean)
% Programmende
